@@ -245,6 +245,11 @@ public class ThreadLocalFury extends AbstractThreadSafeFury {
     return bindingThreadLocal.get().get().deserializeJavaObjectAndClass(channel);
   }
 
+  @Override
+  public <T> T copy(T obj) {
+    return bindingThreadLocal.get().get().copy(obj);
+  }
+
   @Override
   public void setClassLoader(ClassLoader classLoader) {
     setClassLoader(classLoader, StagingType.STRONG_STAGING);
