@@ -76,6 +76,7 @@ import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Supplier;
 import java.util.stream.Collectors;
 import org.apache.fury.Fury;
+import org.apache.fury.FuryCopyable;
 import org.apache.fury.annotation.CodegenInvoke;
 import org.apache.fury.annotation.Internal;
 import org.apache.fury.builder.CodecUtils;
@@ -110,6 +111,7 @@ import org.apache.fury.serializer.CodegenSerializer.LazyInitBeanSerializer;
 import org.apache.fury.serializer.CompatibleSerializer;
 import org.apache.fury.serializer.EnumSerializer;
 import org.apache.fury.serializer.ExternalizableSerializer;
+import org.apache.fury.serializer.FuryCopyableSerializer;
 import org.apache.fury.serializer.JavaSerializer;
 import org.apache.fury.serializer.JdkProxySerializer;
 import org.apache.fury.serializer.LambdaSerializer;
@@ -807,7 +809,7 @@ public class ClassResolver {
   }
 
   public Class<? extends Serializer> getSerializerClass(Class<?> cls, boolean codegen) {
-    if (ReflectionUtils.isAbstract(cls) || cls.isInterface()) {
+    if (!cls.isEnum() && (ReflectionUtils.isAbstract(cls) || cls.isInterface())) {
       throw new UnsupportedOperationException(
           String.format("Class %s doesn't support serialization.", cls));
     }
@@ -1005,7 +1007,7 @@ public class ClassResolver {
         }
       }
     } else {
-      if (fury.getConfig().isCodeGenEnabled()) {
+      if (codegen) {
         LOG.info("Object of type {} can't be serialized by jit", cls);
       }
       switch (fury.getCompatibleMode()) {
@@ -1198,7 +1200,11 @@ public class ClassResolver {
     }
 
     Class<? extends Serializer> serializerClass = getSerializerClass(cls);
-    return Serializers.newSerializer(fury, cls, serializerClass);
+    Serializer serializer = Serializers.newSerializer(fury, cls, serializerClass);
+    if (FuryCopyable.class.isAssignableFrom(cls)) {
+      serializer = new FuryCopyableSerializer<>(fury, cls, serializer);
+    }
+    return serializer;
   }
 
   private String generateSecurityMsg(Class<?> cls) {
@@ -1464,9 +1470,6 @@ public class ClassResolver {
    */
   public void readClassDefs(MemoryBuffer buffer) {
     MetaContext metaContext = fury.getSerializationContext().getMetaContext();
-    int classDefOffset = buffer.readInt32();
-    int readerIndex = buffer.readerIndex();
-    buffer.readerIndex(classDefOffset);
     int numClassDefs = buffer.readVarUint32Small14();
     for (int i = 0; i < numClassDefs; i++) {
       long id = buffer.readInt64();
@@ -1485,7 +1488,6 @@ public class ClassResolver {
       // can be created still.
       metaContext.readClassInfos.add(null);
     }
-    buffer.readerIndex(readerIndex);
   }
 
   private Tuple2<ClassDef, ClassInfo> readClassDef(MemoryBuffer buffer, long header) {
