@@ -65,6 +65,13 @@ public abstract class AbstractCollectionSerializer<T> extends Serializer<T> {
     elementClassInfoHolder = fury.getClassResolver().nilClassInfoHolder();
   }
 
+  public AbstractCollectionSerializer(
+      Fury fury, Class<T> cls, boolean supportCodegenHook, boolean immutable) {
+    super(fury, cls, immutable);
+    this.supportCodegenHook = supportCodegenHook;
+    elementClassInfoHolder = fury.getClassResolver().nilClassInfoHolder();
+  }
+
   private GenericType getElementGenericType(Fury fury) {
     GenericType genericType = fury.getGenerics().nextGenericType();
     GenericType elemGenericType = null;
@@ -507,6 +514,34 @@ public abstract class AbstractCollectionSerializer<T> extends Serializer<T> {
     }
   }
 
+  public Collection newCollection(Collection collection) {
+    numElements = collection.size();
+    return newCollection();
+  }
+
+  /**
+   * Collection must have default constructor to be invoked by fury, otherwise created object can't
+   * be used to adding elements. For example:
+   *
+   * <pre>{@code new ArrayList<Integer> {add(1);}}</pre>
+   *
+   * <p>without default constructor, created list will have elementData as null, adding elements
+   * will raise NPE.
+   *
+   * @return empty collection instance
+   */
+  public Collection newCollection() {
+    if (constructor == null) {
+      constructor = ReflectionUtils.getCtrHandle(type, true);
+    }
+    try {
+      return (Collection) constructor.invoke();
+    } catch (Throwable e) {
+      // reduce code size of critical path.
+      throw buildException(e);
+    }
+  }
+
   private RuntimeException buildException(Throwable e) {
     return new IllegalArgumentException(
         "Please provide public no arguments constructor for class " + type, e);
@@ -514,8 +549,8 @@ public abstract class AbstractCollectionSerializer<T> extends Serializer<T> {
 
   /**
    * Get and reset numElements of deserializing collection. Should be called after {@link
-   * #newCollection}. Nested read may overwrite this element, reset is necessary to avoid use wrong
-   * value by mistake.
+   * #newCollection(MemoryBuffer buffer)}. Nested read may overwrite this element, reset is
+   * necessary to avoid use wrong value by mistake.
    */
   public int getAndClearNumElements() {
     int size = numElements;
