@@ -35,6 +35,7 @@ import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.PriorityQueue;
 import java.util.Set;
 import java.util.SortedSet;
@@ -101,6 +102,28 @@ public class CollectionSerializers {
       super(fury, cls, false);
     }
 
+    @Override
+    public List<?> copy(List<?> originCollection) {
+      Object[] elements = new Object[originCollection.size()];
+      List<?> newCollection = Arrays.asList(elements);
+      if (needToCopyRef) {
+        List<?> copyObject = (List<?>) fury.getCopyObject(originCollection);
+        if (copyObject != null) {
+          return copyObject;
+        }
+        fury.reference(originCollection, newCollection);
+      }
+      copyElements(originCollection, elements);
+      return newCollection;
+    }
+
+    private void copyElements(List<?> originCollection, Object[] elements) {
+      int size = originCollection.size();
+      for (int i = 0; i < size; i++) {
+        elements[i] = fury.copyObject(originCollection.get(i));
+      }
+    }
+
     @Override
     public short getXtypeId() {
       return (short) -Type.LIST.getId();
@@ -221,6 +244,22 @@ public class CollectionSerializers {
       fury.getRefResolver().reference(collection);
       return collection;
     }
+
+    @Override
+    public Collection newCollection(Collection originCollection) {
+      Collection collection;
+      Comparator comparator = fury.copyObject(((SortedSet) originCollection).comparator());
+      if (Objects.equals(type, TreeSet.class)) {
+        collection = new TreeSet(comparator);
+      } else {
+        try {
+          collection = (T) constructor.invoke(comparator);
+        } catch (Throwable e) {
+          throw new RuntimeException(e);
+        }
+      }
+      return collection;
+    }
   }
 
   // ------------------------------ collections serializers ------------------------------ //
@@ -231,7 +270,7 @@ public class CollectionSerializers {
   public static final class EmptyListSerializer extends CollectionSerializer<List<?>> {
 
     public EmptyListSerializer(Fury fury, Class<List<?>> cls) {
-      super(fury, cls, false);
+      super(fury, cls, false, true);
     }
 
     @Override
@@ -284,7 +323,7 @@ public class CollectionSerializers {
   public static final class EmptySetSerializer extends CollectionSerializer<Set<?>> {
 
     public EmptySetSerializer(Fury fury, Class<Set<?>> cls) {
-      super(fury, cls, false);
+      super(fury, cls, false, true);
     }
 
     @Override
@@ -316,7 +355,7 @@ public class CollectionSerializers {
   public static final class EmptySortedSetSerializer extends CollectionSerializer<SortedSet<?>> {
 
     public EmptySortedSetSerializer(Fury fury, Class<SortedSet<?>> cls) {
-      super(fury, cls, false);
+      super(fury, cls, false, true);
     }
 
     @Override
@@ -335,6 +374,11 @@ public class CollectionSerializers {
       super(fury, cls, false);
     }
 
+    @Override
+    public List<?> copy(List<?> originCollection) {
+      return Collections.singletonList(fury.copyObject(originCollection.get(0)));
+    }
+
     @Override
     public void write(MemoryBuffer buffer, List<?> value) {
       fury.writeRef(buffer, value.get(0));
@@ -369,6 +413,11 @@ public class CollectionSerializers {
       super(fury, cls, false);
     }
 
+    @Override
+    public Set<?> copy(Set<?> originCollection) {
+      return Collections.singleton(fury.copyObject(originCollection.iterator().next()));
+    }
+
     @Override
     public void write(MemoryBuffer buffer, Set<?> value) {
       fury.writeRef(buffer, value.iterator().next());
@@ -445,6 +494,16 @@ public class CollectionSerializers {
       return set;
     }
 
+    @Override
+    public Collection newCollection(Collection originCollection) {
+      Map<?, Boolean> map =
+          (Map<?, Boolean>) Platform.getObject(originCollection, MAP_FIELD_OFFSET);
+      AbstractMapSerializer mapSerializer =
+          (AbstractMapSerializer) fury.getClassResolver().getSerializer(map.getClass());
+      Map newMap = mapSerializer.newMap();
+      return Collections.newSetFromMap(newMap);
+    }
+
     @Override
     public Collection onCollectionWrite(MemoryBuffer buffer, Set<?> value) {
       final Map<?, Boolean> map = (Map<?, Boolean>) Platform.getObject(value, MAP_FIELD_OFFSET);
@@ -471,6 +530,11 @@ public class CollectionSerializers {
       fury.getRefResolver().reference(keySetView);
       return keySetView;
     }
+
+    @Override
+    public Collection newCollection(Collection collection) {
+      return ConcurrentHashMap.newKeySet(collection.size());
+    }
   }
 
   public static final class VectorSerializer extends CollectionSerializer<Vector> {
@@ -543,6 +607,11 @@ public class CollectionSerializers {
       }
       return object;
     }
+
+    @Override
+    public EnumSet copy(EnumSet originCollection) {
+      return EnumSet.copyOf(originCollection);
+    }
   }
 
   public static class BitSetSerializer extends Serializer<BitSet> {
@@ -557,6 +626,11 @@ public class CollectionSerializers {
           values, Platform.LONG_ARRAY_OFFSET, Math.multiplyExact(values.length, 8));
     }
 
+    @Override
+    public BitSet copy(BitSet originCollection) {
+      return BitSet.valueOf(originCollection.toLongArray());
+    }
+
     @Override
     public BitSet read(MemoryBuffer buffer) {
       long[] values = buffer.readLongs(buffer.readVarUint32Small7());
@@ -575,6 +649,12 @@ public class CollectionSerializers {
       return value;
     }
 
+    @Override
+    public Collection newCollection(Collection collection) {
+      return new PriorityQueue(
+          collection.size(), fury.copyObject(((PriorityQueue) collection).comparator()));
+    }
+
     @Override
     public PriorityQueue newCollection(MemoryBuffer buffer) {
       int numElements = buffer.readVarUint32Small7();
@@ -625,6 +705,11 @@ public class CollectionSerializers {
       dataSerializer.write(buffer, value);
     }
 
+    @Override
+    public T copy(T originCollection) {
+      return fury.copyObject(originCollection, dataSerializer);
+    }
+
     @Override
     public T read(MemoryBuffer buffer) {
       return dataSerializer.read(buffer);
@@ -667,6 +752,11 @@ public class CollectionSerializers {
     public void write(MemoryBuffer buffer, T value) {
       serializer.write(buffer, value);
     }
+
+    @Override
+    public T copy(T value) {
+      return fury.copyObject(value, (Serializer<T>) serializer);
+    }
   }
 
   // TODO add JDK11:JdkImmutableListSerializer,JdkImmutableMapSerializer,JdkImmutableSetSerializer
