@@ -221,6 +221,16 @@ public class GuavaCollectionSerializers {
       return new MapContainer(numElements);
     }
 
+    @Override
+    public Map newMap(Map map) {
+      return new MapContainer(map.size());
+    }
+
+    @Override
+    public T onMapCopy(Map map) {
+      return onMapRead(map);
+    }
+
     @Override
     public T onMapRead(Map map) {
       MapContainer container = (MapContainer) map;
@@ -341,6 +351,17 @@ public class GuavaCollectionSerializers {
       return new SortedMapContainer<>(comparator, numElements);
     }
 
+    @Override
+    public Map newMap(Map map) {
+      Comparator comparator = fury.copyObject(((ImmutableSortedMap) map).comparator());
+      return new SortedMapContainer<>(comparator, map.size());
+    }
+
+    @Override
+    public T onMapCopy(Map map) {
+      return onMapRead(map);
+    }
+
     @Override
     public T onMapRead(Map map) {
       SortedMapContainer mapContainer = (SortedMapContainer) map;
