@@ -21,9 +21,12 @@ package org.apache.fury.serializer;
 
 import static org.testng.Assert.*;
 
+import lombok.AllArgsConstructor;
+import lombok.Data;
 import org.apache.fury.Fury;
 import org.apache.fury.FuryTestBase;
 import org.apache.fury.codegen.JaninoUtils;
+import org.apache.fury.config.CompatibleMode;
 import org.apache.fury.config.FuryBuilder;
 import org.apache.fury.config.Language;
 import org.testng.annotations.Test;
@@ -60,15 +63,13 @@ public class EnumSerializerTest extends FuryTestBase {
             .requireClassRegistration(false);
     Fury fury1 = builder.build();
     Fury fury2 = builder.build();
-    assertEquals(EnumSerializerTest.EnumFoo.A, serDe(fury1, fury2, EnumSerializerTest.EnumFoo.A));
-    assertEquals(EnumSerializerTest.EnumFoo.B, serDe(fury1, fury2, EnumSerializerTest.EnumFoo.B));
-    assertEquals(
-        EnumSerializerTest.EnumSubClass.A, serDe(fury1, fury2, EnumSerializerTest.EnumSubClass.A));
-    assertEquals(
-        EnumSerializerTest.EnumSubClass.B, serDe(fury1, fury2, EnumSerializerTest.EnumSubClass.B));
+    assertEquals(EnumFoo.A, serDe(fury1, fury2, EnumFoo.A));
+    assertEquals(EnumFoo.B, serDe(fury1, fury2, EnumFoo.B));
+    assertEquals(EnumSubClass.A, serDe(fury1, fury2, EnumSubClass.A));
+    assertEquals(EnumSubClass.B, serDe(fury1, fury2, EnumSubClass.B));
   }
 
-  @Test()
+  @Test
   public void testEnumSerializationUnexistentEnumValueAsNull() {
     String enumCode2 = "enum TestEnum2 {" + " A;" + "}";
     String enumCode1 = "enum TestEnum2 {" + " A, B" + "}";
@@ -93,4 +94,26 @@ public class EnumSerializerTest extends FuryTestBase {
     byte[] bytes = furySerialization.serialize(cls1.getEnumConstants()[1]);
     Object data = furyDeserialize.deserialize(bytes);
   }
+
+  @Data
+  @AllArgsConstructor
+  static class EnumSubclassFieldTest {
+    EnumSubClass subEnum;
+  }
+
+  @Test(dataProvider = "enableCodegen")
+  public void testEnumSubclassField(boolean enableCodegen) {
+    serDeCheck(
+        builder().withCodegen(enableCodegen).build(), new EnumSubclassFieldTest(EnumSubClass.B));
+  }
+
+  @Test(dataProvider = "scopedMetaShare")
+  public void testEnumSubclassFieldCompatible(boolean scopedMetaShare) {
+    serDeCheck(
+        builder()
+            .withScopedMetaShare(scopedMetaShare)
+            .withCompatibleMode(CompatibleMode.COMPATIBLE)
+            .build(),
+        new EnumSubclassFieldTest(EnumSubClass.B));
+  }
 }
