@@ -51,7 +51,9 @@ import java.util.SortedSet;
 import java.util.TreeSet;
 import java.util.Vector;
 import java.util.concurrent.ArrayBlockingQueue;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentSkipListSet;
+import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.stream.LongStream;
 import lombok.AllArgsConstructor;
@@ -66,6 +68,7 @@ import org.apache.fury.serializer.collection.CollectionSerializers.JDKCompatible
 import org.apache.fury.type.GenericType;
 import org.testng.Assert;
 import org.testng.annotations.Test;
+import org.testng.collections.Maps;
 
 public class CollectionSerializersTest extends FuryTestBase {
 
@@ -263,6 +266,46 @@ public class CollectionSerializersTest extends FuryTestBase {
         CollectionSerializers.PriorityQueueSerializer.class);
   }
 
+  @Test
+  public void testCopyOnWriteArrayList() {
+    final CopyOnWriteArrayList<String> list =
+        new CopyOnWriteArrayList<>(new String[] {"a", "b", "c"});
+    Assert.assertEquals(list, serDe(getJavaFury(), list));
+    Assert.assertEquals(
+        getJavaFury().getClassResolver().getSerializerClass(CopyOnWriteArrayList.class),
+        CollectionSerializers.CopyOnWriteArrayListSerializer.class);
+  }
+
+  @Test(dataProvider = "enableCodegen")
+  public void testSetFromMap(boolean enableCodegen) {
+    final Fury fury =
+        Fury.builder()
+            .withLanguage(Language.JAVA)
+            .requireClassRegistration(false)
+            .withCodegen(enableCodegen)
+            .build();
+    final Set<String> set = Collections.newSetFromMap(Maps.newConcurrentMap());
+    set.add("a");
+    set.add("b");
+    set.add("c");
+    Assert.assertEquals(set, serDe(fury, set));
+    Assert.assertEquals(
+        getJavaFury().getClassResolver().getSerializerClass(set.getClass()),
+        CollectionSerializers.SetFromMapSerializer.class);
+  }
+
+  @Test
+  public void testConcurrentMapKeySetViewMap() {
+    final ConcurrentHashMap.KeySetView<Object, Boolean> set = ConcurrentHashMap.newKeySet();
+    set.add("a");
+    set.add("b");
+    set.add("c");
+    Assert.assertEquals(set, serDe(getJavaFury(), set));
+    Assert.assertEquals(
+        getJavaFury().getClassResolver().getSerializerClass(set.getClass()),
+        CollectionSerializers.ConcurrentHashMapKeySetView.class);
+  }
+
   @Test
   public void testSerializeJavaBlockingQueue() {
     Fury fury =
@@ -574,4 +617,31 @@ public class CollectionSerializersTest extends FuryTestBase {
     Fury f = Fury.builder().withLanguage(Language.JAVA).withRefTracking(refTracking).build();
     serDeCheck(f, data);
   }
+
+  @Data
+  abstract static class Foo {
+    private int f1;
+  }
+
+  static class Foo1 extends Foo {}
+
+  @Data
+  static class CollectionAbstractTest {
+    private List<Foo> fooList;
+  }
+
+  @Test(dataProvider = "enableCodegen")
+  public void testAbstractCollectionElementsSerialization(boolean enableCodegen) {
+    Fury fury = Fury.builder().withCodegen(enableCodegen).requireClassRegistration(false).build();
+    {
+      CollectionAbstractTest test = new CollectionAbstractTest();
+      test.fooList = new ArrayList<>(ImmutableList.of(new Foo1(), new Foo1()));
+      serDeCheck(fury, test);
+    }
+    {
+      CollectionAbstractTest test = new CollectionAbstractTest();
+      test.fooList = new ArrayList<>(ofArrayList(new Foo1(), new Foo1()));
+      serDeCheck(fury, test);
+    }
+  }
 }
